#include<vcg/complex/complex.h>
#include<vcg/complex/algorithms/create/platonic.h>
#include<vcg/space/line2.h>
#include<vcg/space/intersection2.h>
#include<wrap/io_trimesh/import_off.h> 
#include<wrap/io_trimesh/export_off.h> 
#include<wrap/io_trimesh/export_ply.h> 

using namespace vcg;

class MyVertex; class MyEdge; class MyFace;
struct MyUsedTypes : public UsedTypes<Use<MyVertex>   ::AsVertexType,
                                           Use<MyEdge>     ::AsEdgeType,
                                           Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public Vertex< MyUsedTypes, vertex::Coord3f, vertex::Normal3f, vertex::BitFlags  >{};
class MyFace    : public Face<   MyUsedTypes, face::FFAdj, face::Normal3f,  face::VertexRef, face::BitFlags > {};
class MyEdge    : public Edge<   MyUsedTypes> {};

class MyMesh    : public tri::TriMesh< std::vector<MyVertex>, 
    std::vector<MyFace> , std::vector<MyEdge>  > {};

int main( int /*argc*/, char **argv )
{
  MyMesh m;
  MyMesh out;
  
  tri::io::ImporterOFF<MyMesh>::Open(m,argv[1]);
  tri::UpdateNormal<MyMesh>::PerVertexNormalizedPerFaceNormalized(m);
  printf("Input mesh  vn:%i fn:%i\n",m.VN(),m.FN());
  for(int i=0;i<10000;++i)
  {
    MyFace &f=m.face[rand()%m.FN()];
    if(f.N().Z()>0)
    {
      Point3f p = (f.P(0)+f.P(1)+f.P(2))/3.0f;  
      Point3f d = f.N(); d[2]=0;
      Point3f l = f.N() ^ d;
      Point3f t = l ^f.N();    // t is the vector that lies on the plane of the triangle f and is oriented along the maximum z-decrease 
      t.Normalize();
      
      Point2f tp2[3]; // 2D positions of the vertexes of the triangle projected on the XY plane
      Line2f e[3];    // lines on the 2D plane passing through the edges 
      
      for(int j=0;j<3;++j)
        tp2[j] = Point2f(f.P(j).X(),f.P(j).Y());
      
      Point2f p2 = Point2f(p[0],p[1]);  // the midpoint from which we should start
      Point2f t2 = Point2f(t[0],t[1]);  // the projection of the t vector onto the XY plane
      Line2f dir(p2,t2);                // 2D line from point along the t direction 
      
      bool res[3];
      Point2f ip[3];   // intersections of the edge lines with the line dir
      int bestSide = -1;
      float minDist = std::numeric_limits<float>::max();
      for(int j=0;j<3;++j)
      {
        e[j] = Line2f(tp2[j],tp2[(j+1)%3]-tp2[j]);
        res[j] = LineLineIntersection(dir,e[j],ip[j]);
        if(res[j])
        {
          Point2f dd = ip[j]-p2; 
          if(dd*t2 >0)
          {
            if(Distance(ip[j],p2) < minDist)
            {
              minDist = Distance(ip[j],p2);
              bestSide=j;
            }            
          }
        }
      }
      assert(bestSide != -1);
      
      // riportare in 3D l'intersezione 2D tra la linea e gli edge.     
      Point3f newp(ip[bestSide][0],ip[bestSide][1],p[2]); 
      if(fabs(t[0])> fabs(t[1]))
        newp[2] = p[2] + t[2]*(ip[bestSide][0] - p[0])/t[0];
      else
        newp[2] = p[2] + t[2]*(ip[bestSide][1] - p[1])/t[1];
      
      tri::Allocator<MyMesh>::AddVertex(out,p,t); 
      tri::Allocator<MyMesh>::AddVertex(out,newp,newp); 
    }
  }
  
  tri::io::ExporterPLY<MyMesh>::Save(out,"out.ply",tri::io::Mask::IOM_VERTNORMAL);
  return 0;
}
